@@ -951,35 +951,32 @@ GetUTCTime(JSContext *cx, JSObject *obj, jsval *vp, jsdouble *dp)
     return JS_TRUE;
 }
 
-/*
- * Set UTC time slot with a pointer pointing to a jsdouble. This function is
- * used only for setting UTC time to some predefined values, such as NaN.
- *
- * It also invalidates cached local time.
- */
-static JSBool
-SetUTCTimePtr(JSContext *cx, JSObject *obj, jsval *vp, jsdouble *dp)
+static void
+SetDateToNaN(JSContext *cx, JSObject *obj, jsval *vp = NULL)
 {
-    if (vp && !JS_InstanceOf(cx, obj, &js_DateClass, vp + 2))
-        return JS_FALSE;
-    JS_ASSERT_IF(!vp, STOBJ_GET_CLASS(obj) == &js_DateClass);
+    JS_ASSERT(OBJ_GET_CLASS(cx, obj) == &js_DateClass);
 
-    /* Invalidate local time cache. */
-    obj->fslots[JSSLOT_LOCAL_TIME] = DOUBLE_TO_JSVAL(cx->runtime->jsNaN);
-    obj->fslots[JSSLOT_UTC_TIME] = DOUBLE_TO_JSVAL(dp);
-    return JS_TRUE;
+    jsval nan = DOUBLE_TO_JSVAL(cx->runtime->jsNaN);
+    obj->fslots[JSSLOT_LOCAL_TIME] = nan;
+    obj->fslots[JSSLOT_UTC_TIME] = nan;
+    if (vp)
+        *vp = nan;
 }
 
 /*
- * Set UTC time to a given time.
+ * Set UTC time to a given time and invalidate cached local time.
  */
 static JSBool
-SetUTCTime(JSContext *cx, JSObject *obj, jsval *vp, jsdouble t)
+SetUTCTime(JSContext *cx, JSObject *obj, jsdouble t, jsval *vp = NULL)
 {
-    jsdouble *dp = js_NewWeaklyRootedDouble(cx, t);
-    if (!dp)
-        return JS_FALSE;
-    return SetUTCTimePtr(cx, obj, vp, dp);
+    JS_ASSERT(OBJ_GET_CLASS(cx, obj) == &js_DateClass);
+
+    obj->fslots[JSSLOT_LOCAL_TIME] = DOUBLE_TO_JSVAL(cx->runtime->jsNaN);
+    if (!js_NewDoubleInRootedValue(cx, t, &obj->fslots[JSSLOT_UTC_TIME]))
+        return false;
+    if (vp)
+        *vp = obj->fslots[JSSLOT_UTC_TIME];
+    return true;
 }
 
 /*
@@ -1268,35 +1265,23 @@ date_getTimezoneOffset(JSContext *cx, uintN argc, jsval *vp)
     return js_NewNumberInRootedValue(cx, result, vp);
 }
 
-static JSBool
-SetDateToNaN(JSContext *cx, jsval *vp)
-{
-    JSObject *obj;
-
-    obj = JS_THIS_OBJECT(cx, vp);
-    if (!SetUTCTimePtr(cx, obj, NULL, cx->runtime->jsNaN))
-        return JS_FALSE;
-    *vp = DOUBLE_TO_JSVAL(cx->runtime->jsNaN);
-    return JS_TRUE;
-}
-
 static JSBool
 date_setTime(JSContext *cx, uintN argc, jsval *vp)
 {
-    jsdouble result;
+    JSObject *obj = JS_THIS_OBJECT(cx, vp);
+    if (!JS_InstanceOf(cx, obj, &js_DateClass, vp + 2))
+        return false;
 
-    if (argc == 0)
-        return SetDateToNaN(cx, vp);
-    result = js_ValueToNumber(cx, &vp[2]);
+    if (argc == 0) {
+        SetDateToNaN(cx, obj, vp);
+        return true;
+    }
+
+    jsdouble result = js_ValueToNumber(cx, &vp[2]);
     if (JSVAL_IS_NULL(vp[2]))
-        return JS_FALSE;
+        return false;
 
-    result = TIMECLIP(result);
-
-    if (!SetUTCTime(cx, JS_THIS_OBJECT(cx, vp), vp, result))
-        return JS_FALSE;
-
-    return js_NewNumberInRootedValue(cx, result, vp);
+    return SetUTCTime(cx, obj, TIMECLIP(result), vp);
 }
 
 static JSBool
@@ -1314,7 +1299,7 @@ date_makeTime(JSContext *cx, uintN maxargs, JSBool local, uintN argc, jsval *vp)
 
     obj = JS_THIS_OBJECT(cx, vp);
     if (!GetUTCTime(cx, obj, vp, &result))
-        return JS_FALSE;
+        return false;
 
     /* just return NaN if the date is already NaN */
     if (!JSDOUBLE_IS_FINITE(result))
@@ -1329,8 +1314,10 @@ date_makeTime(JSContext *cx, uintN maxargs, JSBool local, uintN argc, jsval *vp)
      * if it's not given.  This means that "d = new Date();
      * d.setMilliseconds()" returns NaN.  Blech.
      */
-    if (argc == 0)
-        return SetDateToNaN(cx, vp);
+    if (argc == 0) {
+        SetDateToNaN(cx, obj, vp);
+        return true;
+    }
     if (argc > maxargs)
         argc = maxargs;  /* clamp argc */
     JS_ASSERT(argc <= 4);
@@ -1340,8 +1327,10 @@ date_makeTime(JSContext *cx, uintN maxargs, JSBool local, uintN argc, jsval *vp)
         args[i] = js_ValueToNumber(cx, &argv[i]);
         if (JSVAL_IS_NULL(argv[i]))
             return JS_FALSE;
-        if (!JSDOUBLE_IS_FINITE(args[i]))
-            return SetDateToNaN(cx, vp);
+        if (!JSDOUBLE_IS_FINITE(args[i])) {
+            SetDateToNaN(cx, obj, vp);
+            return true;
+        }
         args[i] = js_DoubleToInteger(args[i]);
     }
 
@@ -1382,11 +1371,7 @@ date_makeTime(JSContext *cx, uintN maxargs, JSBool local, uintN argc, jsval *vp)
 
 /*     fprintf(stderr, "%f\n", result); */
 
-    result = TIMECLIP(result);
-    if (!SetUTCTime(cx, obj, NULL, result))
-        return JS_FALSE;
-
-    return js_NewNumberInRootedValue(cx, result, vp);
+    return SetUTCTime(cx, obj, TIMECLIP(result), vp);
 }
 
 static JSBool
@@ -1450,11 +1435,13 @@ date_makeDate(JSContext *cx, uintN maxargs, JSBool local, uintN argc, jsval *vp)
 
     obj = JS_THIS_OBJECT(cx, vp);
     if (!GetUTCTime(cx, obj, vp, &result))
-        return JS_FALSE;
+        return false;
 
     /* see complaint about ECMA in date_MakeTime */
-    if (argc == 0)
-        return SetDateToNaN(cx, vp);
+    if (argc == 0) {
+        SetDateToNaN(cx, obj, vp);
+        return true;
+    }
     if (argc > maxargs)
         argc = maxargs;   /* clamp argc */
     JS_ASSERT(1 <= argc && argc <= 3);
@@ -1463,9 +1450,11 @@ date_makeDate(JSContext *cx, uintN maxargs, JSBool local, uintN argc, jsval *vp)
     for (i = 0; i < argc; i++) {
         args[i] = js_ValueToNumber(cx, &argv[i]);
         if (JSVAL_IS_NULL(argv[i]))
-            return JS_FALSE;
-        if (!JSDOUBLE_IS_FINITE(args[i]))
-            return SetDateToNaN(cx, vp);
+            return false;
+        if (!JSDOUBLE_IS_FINITE(args[i])) {
+            SetDateToNaN(cx, obj, vp);
+            return true;
+        }
         args[i] = js_DoubleToInteger(args[i]);
     }
 
@@ -1502,11 +1491,7 @@ date_makeDate(JSContext *cx, uintN maxargs, JSBool local, uintN argc, jsval *vp)
     if (local)
         result = UTC(result);
 
-    result = TIMECLIP(result);
-    if (!SetUTCTime(cx, obj, NULL, result))
-        return JS_FALSE;
-
-    return js_NewNumberInRootedValue(cx, result, vp);
+    return SetUTCTime(cx, obj, TIMECLIP(result), vp);
 }
 
 static JSBool
@@ -1548,44 +1533,35 @@ date_setUTCFullYear(JSContext *cx, uintN argc, jsval *vp)
 static JSBool
 date_setYear(JSContext *cx, uintN argc, jsval *vp)
 {
-    JSObject *obj;
-    jsdouble t;
-    jsdouble year;
-    jsdouble day;
+    JSObject *obj = JS_THIS_OBJECT(cx, vp);
+
     jsdouble result;
-
-    obj = JS_THIS_OBJECT(cx, vp);
     if (!GetUTCTime(cx, obj, vp, &result))
-        return JS_FALSE;
+        return false;
 
-    if (argc == 0)
-        return SetDateToNaN(cx, vp);
-    year = js_ValueToNumber(cx, &vp[2]);
-    if (JSVAL_IS_NULL(vp[2]))
-        return JS_FALSE;
-    if (!JSDOUBLE_IS_FINITE(year))
-        return SetDateToNaN(cx, vp);
-
-    year = js_DoubleToInteger(year);
-
-    if (!JSDOUBLE_IS_FINITE(result)) {
-        t = +0.0;
-    } else {
-        t = LocalTime(result);
+    if (argc == 0) {
+        /* Call this only after GetUTCTime has verified that obj is Date. */
+        SetDateToNaN(cx, obj, vp);
+        return true;
     }
 
+    jsdouble year = js_ValueToNumber(cx, &vp[2]);
+    if (JSVAL_IS_NULL(vp[2]))
+        return false;
+    if (!JSDOUBLE_IS_FINITE(year)) {
+        SetDateToNaN(cx, obj, vp);
+        return true;
+    }
+    year = js_DoubleToInteger(year);
     if (year >= 0 && year <= 99)
         year += 1900;
 
-    day = MakeDay(year, MonthFromTime(t), DateFromTime(t));
+    jsdouble t = JSDOUBLE_IS_FINITE(result) ? LocalTime(result) : +0.0;
+    jsdouble day = MakeDay(year, MonthFromTime(t), DateFromTime(t));
     result = MakeDate(day, TimeWithinDay(t));
     result = UTC(result);
 
-    result = TIMECLIP(result);
-    if (!SetUTCTime(cx, obj, NULL, result))
-        return JS_FALSE;
-
-    return js_NewNumberInRootedValue(cx, result, vp);
+    return SetUTCTime(cx, obj, TIMECLIP(result), vp);
 }
 
 /* constants for toString, toUTCString */
@@ -2091,87 +2067,54 @@ static JSFunctionSpec date_methods[] = {
     JS_FS_END
 };
 
-static jsdouble *
-date_constructor(JSContext *cx, JSObject* obj)
-{
-    jsdouble *date;
-
-    date = js_NewWeaklyRootedDouble(cx, 0.0);
-    if (!date)
-        return NULL;
-
-    obj->fslots[JSSLOT_UTC_TIME] = DOUBLE_TO_JSVAL(date);
-    obj->fslots[JSSLOT_LOCAL_TIME] = DOUBLE_TO_JSVAL(cx->runtime->jsNaN);
-    return date;
-}
-
 JSBool
 js_Date(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
-    jsdouble *date;
-    JSString *str;
-    jsdouble d;
-
     /* Date called as function. */
     if (!JS_IsConstructing(cx))
         return date_format(cx, NowAsMillis(), FORMATSPEC_FULL, rval);
 
     /* Date called as constructor. */
+    jsdouble d;
     if (argc == 0) {
-        date = date_constructor(cx, obj);
-        if (!date)
-            return JS_FALSE;
-        *date = NowAsMillis();
+        d = NowAsMillis();
     } else if (argc == 1) {
         if (!JSVAL_IS_STRING(argv[0])) {
             /* the argument is a millisecond number */
             d = js_ValueToNumber(cx, &argv[0]);
             if (JSVAL_IS_NULL(argv[0]))
                 return JS_FALSE;
-            date = date_constructor(cx, obj);
-            if (!date)
-                return JS_FALSE;
-            *date = TIMECLIP(d);
+            d = TIMECLIP(d);
         } else {
             /* the argument is a string; parse it. */
-            date = date_constructor(cx, obj);
-            if (!date)
-                return JS_FALSE;
-
-            str = js_ValueToString(cx, argv[0]);
+            JSString *str = js_ValueToString(cx, argv[0]);
             if (!str)
                 return JS_FALSE;
+            argv[0] = STRING_TO_JSVAL(str);
 
-            if (!date_parseString(str, date))
-                *date = *cx->runtime->jsNaN;
-            *date = TIMECLIP(*date);
+            if (!date_parseString(str, &d))
+                d = js_NaN;
+            else
+                d = TIMECLIP(d);
         }
     } else {
-        jsdouble *date;
         jsdouble msec_time;
-
         if (!date_msecFromArgs(cx, argc, argv, &msec_time))
             return JS_FALSE;
 
-        date = date_constructor(cx, obj);
-        if (!date)
-            return JS_FALSE;
-
         if (JSDOUBLE_IS_FINITE(msec_time)) {
             msec_time = UTC(msec_time);
             msec_time = TIMECLIP(msec_time);
         }
-
-        *date = msec_time;
+        d = msec_time;
     }
-    return JS_TRUE;
+    return SetUTCTime(cx, obj, d);
 }
 
 JSObject *
 js_InitDateClass(JSContext *cx, JSObject *obj)
 {
     JSObject *proto;
-    jsdouble *proto_date;
 
     /* set static LocalTZA */
     LocalTZA = -(PRMJ_LocalGMTDifference() * msPerSecond);
@@ -2180,34 +2123,21 @@ js_InitDateClass(JSContext *cx, JSObject *obj)
     if (!proto)
         return NULL;
 
-    /* Alias toUTCString with toGMTString.  (ECMA B.2.6) */
+    SetDateToNaN(cx, proto);
+    
+   /* Alias toUTCString with toGMTString.  (ECMA B.2.6) */
     if (!JS_AliasProperty(cx, proto, "toUTCString", "toGMTString"))
         return NULL;
 
-    /* Set the value of the Date.prototype date to NaN */
-    proto_date = date_constructor(cx, proto);
-    if (!proto_date)
-        return NULL;
-    *proto_date = *cx->runtime->jsNaN;
-
     return proto;
 }
 
 JS_FRIEND_API(JSObject *)
 js_NewDateObjectMsec(JSContext *cx, jsdouble msec_time)
 {
-    JSObject *obj;
-    jsdouble *date;
-
-    obj = js_NewObject(cx, &js_DateClass, NULL, NULL, 0);
-    if (!obj)
+    JSObject *obj = js_NewObject(cx, &js_DateClass, NULL, NULL, 0);
+    if (!obj || !SetUTCTime(cx, obj, msec_time))
         return NULL;
-
-    date = date_constructor(cx, obj);
-    if (!date)
-        return NULL;
-
-    *date = msec_time;
     return obj;
 }
 
@@ -2329,7 +2259,7 @@ js_DateSetYear(JSContext *cx, JSObject *obj, int year)
                               msFromTime(local));
 
     /* SetUTCTime also invalidates local time cache. */
-    SetUTCTime(cx, obj, NULL, UTC(local));
+    SetUTCTime(cx, obj, UTC(local));
 }
 
 JS_FRIEND_API(void)
@@ -2353,7 +2283,7 @@ js_DateSetMonth(JSContext *cx, JSObject *obj, int month)
                               MinFromTime(local),
                               SecFromTime(local),
                               msFromTime(local));
-    SetUTCTime(cx, obj, NULL, UTC(local));
+    SetUTCTime(cx, obj, UTC(local));
 }
 
 JS_FRIEND_API(void)
@@ -2374,7 +2304,7 @@ js_DateSetDate(JSContext *cx, JSObject *obj, int date)
                               MinFromTime(local),
                               SecFromTime(local),
                               msFromTime(local));
-    SetUTCTime(cx, obj, NULL, UTC(local));
+    SetUTCTime(cx, obj, UTC(local));
 }
 
 JS_FRIEND_API(void)
@@ -2394,7 +2324,7 @@ js_DateSetHours(JSContext *cx, JSObject *obj, int hours)
                               MinFromTime(local),
                               SecFromTime(local),
                               msFromTime(local));
-    SetUTCTime(cx, obj, NULL, UTC(local));
+    SetUTCTime(cx, obj, UTC(local));
 }
 
 JS_FRIEND_API(void)
@@ -2414,7 +2344,7 @@ js_DateSetMinutes(JSContext *cx, JSObject *obj, int minutes)
                               minutes,
                               SecFromTime(local),
                               msFromTime(local));
-    SetUTCTime(cx, obj, NULL, UTC(local));
+    SetUTCTime(cx, obj, UTC(local));
 }
 
 JS_FRIEND_API(void)
@@ -2434,7 +2364,7 @@ js_DateSetSeconds(JSContext *cx, JSObject *obj, int seconds)
                               MinFromTime(local),
                               seconds,
                               msFromTime(local));
-    SetUTCTime(cx, obj, NULL, UTC(local));
+    SetUTCTime(cx, obj, UTC(local));
 }
 
 JS_FRIEND_API(jsdouble)
