@@ -16,10 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2008
+ * Portions created by the Initial Developer are Copyright (C) 2005
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Jesse Ruderman
+ * Contributor(s): Andrew Schultz
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,31 +35,18 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-456477-02.js';
+var gTestfile = 'regress-317714-01.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 456477;
-var summary = 'Do not assert with JIT: (m != JSVAL_INT) || isInt32(*vp)" with (0/0)%(-1)';
+var BUGNUMBER = 317714;
+var summary = 'Regression test for regression from bug 316885';
 var actual = 'No Crash';
 var expect = 'No Crash';
 
+var d5="-1";
+var r3=d5.split(":");
+r3[0]++;
 
-//-----------------------------------------------------------------------------
-test();
-//-----------------------------------------------------------------------------
+printBugNumber(BUGNUMBER);
+printStatus (summary);
 
-function test()
-{
-  enterFunc ('test');
-  printBugNumber(BUGNUMBER);
-  printStatus (summary);
- 
-  jit(true);
-
-  (function() { for (var j = 0; j < 5; ++j) { (0 / 0) % (-1); } })();
-
-  jit(false);
-
-  reportCompare(expect, actual, summary);
-
-  exitFunc ('test');
-}
+reportCompare(expect, actual, summary);
