@@ -16,10 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2006
+ * Portions created by the Initial Developer are Copyright (C) 2007
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Igor Bukanov
+ * Contributor(s): Jesse Ruderman
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,10 +35,11 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-350312-01.js';
+var gTestfile = 'regress-358594-05.js';
+
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 350312;
-var summary = 'Accessing wrong stack slot with nested catch/finally';
+var BUGNUMBER = 358594;
+var summary = 'Do not crash on uneval(this).';
 var actual = '';
 var expect = '';
 
@@ -53,31 +54,12 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
 
-  var tmp;
-
-  function f()
-  {
-    try {  
-      try {  
-	throw 1;
-      } catch (e) {
-	throw e;
-      } finally {
-	tmp = true;
-      }
-    } catch (e) {
-      return e;
-    }
-  }
-
-  var ex = f();
-
-  var passed = ex === 1;
-  if (!passed) {
-    print("Failed!");
-    print("ex="+uneval(ex));
-  }
-  reportCompare(true, passed, summary);
+  // don't crash|assert
+  f = function () { };
+  f.hhhhhhhhh = this; 
+  this.m setter = f; 
+  uneval(this);
+  reportCompare(expect, actual, summary);
 
   exitFunc ('test');
 }
