@@ -19,7 +19,7 @@
  * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s):
+ * Contributor(s): Jesse Ruderman
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,10 +35,10 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-452498-099.js';
+var gTestfile = 'regress-469239-01.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 452498;
-var summary = 'TM: upvar2 regression tests';
+var BUGNUMBER = 469239;
+var summary = 'TM: Do not assert: entry->kpc == (jsbytecode*) atoms[index]';
 var actual = '';
 var expect = '';
 
@@ -53,33 +53,17 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
 
-// ------- Comment #99 From Gary Kwong [:nth10sd]
+  jit(true);
 
-  try
-  {
-    eval("(function x(){x.(this)} )();");
-  }
-  catch(ex)
-  {
+  for (let b=0;b<9;++b) {
+    for each (let h in ['', 3, /x/]) {
+	    for each (c in [[], [], [], /x/]) {
+        '' + c;
+	    }
+    }
   }
 
-// Assertion failure: (uint32)(index_) < atoms_->length, at ../jsinterp.cpp:327
-// Crash [@ js_FullTestPropertyCache] at null in opt, -j not required.
-
-// =====
-
-  try
-  {
-    (function(){try {x} finally {}; ([x in []] for each (x in x))})();
-  }
-  catch(ex)
-  {
-  }
-
-// Assertion failure: lexdep->frameLevel() <= funbox->level, at ../jsparse.cpp:1735
-// Crash [@ BindNameToSlot] near null in opt, -j not required.
-
-// =====
+  jit(false);
 
   reportCompare(expect, actual, summary);
 
