@@ -46,6 +46,7 @@
 #include "convert.h"
 
 /*#define DEBUG*/
+#include "debug.h"
 
 #if HAVE_SPEEX
 
@@ -207,10 +208,10 @@ process_header(unsigned char * buf, long bytes, int enh_enabled,
   if (*channels == -1)
     *channels = header->nb_channels;
 
-#ifdef DEBUG
-  fprintf (stderr, "Decoding %d Hz audio using %s mode",
-	   *rate, mode->modeName);
+  debug_printf (1, "Decoding %d Hz audio using %s mode",
+                *rate, mode->modeName);
 
+#ifdef DEBUG
   if (*channels==1)
       fprintf (stderr, " (mono");
    else
@@ -287,9 +288,7 @@ fs_speex_decode (FishSound * fsound, unsigned char * buf, long bytes)
       return FISH_SOUND_ERR_GENERIC;
     }
 
-#ifdef DEBUG
-    printf ("speex: got %d channels, %d Hz\n", channels, rate);
-#endif
+    debug_printf (1, "speex: got %d channels, %d Hz", channels, rate);
 
     fsound->info.samplerate = rate;
     fsound->info.channels = channels;
@@ -436,9 +435,7 @@ fs_speex_enc_headers (FishSound * fsound)
 
   speex_encoder_ctl (fss->st, SPEEX_GET_FRAME_SIZE, &fss->frame_size);
 
-#ifdef DEBUG
-  printf ("got frame size %d\n", fss->frame_size);
-#endif
+  debug_printf (1, "got frame size %d", fss->frame_size);
 
   /* XXX: set VBR etc. */
 
